from __future__ import annotations

from typing import Any

from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.cloudformation.checks.resource.base_resource_value_check import BaseResourceValueCheck


class KMSKeyWildCardPrincipal(BaseResourceValueCheck):
    def __init__(self) -> None:
        name = "Ensure KMS key policy does not contain wildcard (*) principal"
        id = "CKV_AWS_33"
        supported_resources = ("AWS::KMS::Key",)
        categories = (CheckCategories.ENCRYPTION,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self) -> str:
        return "Properties/KeyPolicy/Statement/Principal"

    def scan_resource_conf(self, conf: dict[str, Any]) -> CheckResult:
        properties = conf.get("Properties")
        if properties and isinstance(properties, dict):
            policy_block = properties.get("KeyPolicy")
            if policy_block and isinstance(policy_block, dict):
                statements = policy_block.get("Statement")
                if statements and isinstance(statements, list):
                    for statement in statements:
                        principal = statement.get("Principal")
                        if not principal:
                            continue
                        if statement.get("Effect") == "Deny":
                            continue

                        if isinstance(principal, dict) and "AWS" in principal:
                            # the actual principals can be under the `AWS`
                            principal = principal["AWS"]

                        if isinstance(principal, str) and principal == "*":
                            return CheckResult.FAILED
                        if isinstance(principal, list) and "*" in principal:
                            return CheckResult.FAILED

        return CheckResult.PASSED


check = KMSKeyWildCardPrincipal()



from __future__ import annotations

from typing import Any

from checkov.terraform.checks.resource.base_resource_check import BaseResourceCheck
from checkov.common.models.enums import CheckCategories, CheckResult


class ElasticsearchDomainAuditLogging(BaseResourceCheck):
    def __init__(self) -> None:
        name = "Ensure Elasticsearch Domain Audit Logging is enabled"
        id = "CKV_AWS_317"
        supported_resources = ("aws_elasticsearch_domain", "aws_opensearch_domain")
        categories = (CheckCategories.LOGGING,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf: dict[str, list[Any]]) -> CheckResult:
        options = conf.get("log_publishing_options")
        if options and isinstance(options, list):
            for option in options:
                if isinstance(option, dict):
                    log_type = option.get("log_type")
                    if log_type and isinstance(log_type, list) and log_type[0] == "AUDIT_LOGS":
                        enabled = option.get("enabled")
                        if enabled and isinstance(enabled, list) and enabled[0]:
                            return CheckResult.PASSED

        return CheckResult.FAILED


check = ElasticsearchDomainAuditLogging()



from __future__ import annotations

from checkov.github.base_github_branch_security import BranchSecurity


class GithubBranchRequireStatusChecks(BranchSecurity):
    def __init__(self) -> None:
        name = "Ensure all checks have passed before the merge of new code"
        id = "CKV_GITHUB_14"
        super().__init__(name=name, id=id)

    def get_evaluated_keys(self) -> list[str]:
        return ["required_status_checks"]


check = GithubBranchRequireStatusChecks()



from __future__ import annotations

from typing import Any

from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.github.base_github_configuration_check import BaseGithubCheck, HTTP
from checkov.github.schemas.org_webhooks import schema as org_webhooks_schema
from checkov.json_doc.enums import BlockType


class WebhookHttpsOrg(BaseGithubCheck):
    def __init__(self) -> None:
        name = "Ensure GitHub organization webhooks are using HTTPS"
        id = "CKV_GITHUB_6"
        categories = [CheckCategories.SUPPLY_CHAIN]
        super().__init__(
            id=id,
            name=name,
            categories=categories,
            supported_entities=["*"],
            block_type=BlockType.DOCUMENT
        )

    def scan_entity_conf(self, conf: dict[str, Any], entity_type: str) -> tuple[CheckResult, dict[str, Any]] | None:  # type:ignore[override]
        ckv_metadata, conf = self.resolve_ckv_metadata_conf(conf=conf)
        if 'org_webhooks' in ckv_metadata.get('file_name', ''):
            if org_webhooks_schema.validate(conf):
                for item in conf:
                    if isinstance(item, dict):
                        item_config = item.get("config", {})
                        if not item_config:
                            continue
                        url = item_config.get('url', '')
                        insecure_ssl = item_config.get('insecure_ssl', '0')
                        secret = item_config.get('secret', '')
                        if url.startswith(HTTP):
                            return CheckResult.FAILED, item_config
                        if insecure_ssl != '0' and secret != '********':  # nosec
                            return CheckResult.FAILED, item_config
                return CheckResult.PASSED, conf
        return CheckResult.UNKNOWN, conf


check = WebhookHttpsOrg()



from __future__ import annotations

import logging
from typing import Set, Any, Generator, Pattern, Optional, Dict, Tuple, TYPE_CHECKING, cast

from detect_secrets.constants import VerifiedResult
from detect_secrets.core.potential_secret import PotentialSecret
from detect_secrets.plugins.base import RegexBasedDetector
from detect_secrets.util.inject import call_function_with_arguments
import re

from checkov.common.util.file_utils import read_file_safe, get_file_size_safe
from checkov.secrets.plugins.load_detectors import load_detectors

MIN_CHARACTERS = 5
MAX_CHARACTERS = 100

if TYPE_CHECKING:
    from detect_secrets.util.code_snippet import CodeSnippet


class CustomRegexDetector(RegexBasedDetector):
    secret_type = "Regex Detector"  # noqa: CCE003 # nosec
    denylist: Set[Pattern[str]] = set()  # noqa: CCE003
    MAX_FILE_SIZE: int = 4 * 1024
    MAX_LINE_LENGTH: int = 10_000

    def __init__(self) -> None:
        self.regex_to_metadata: dict[str, dict[str, Any]] = dict()
        self.denylist = set()
        self.multiline_deny_list = set()
        self.multiline_regex_to_metadata: dict[str, dict[str, Any]] = dict()
        self._analyzed_files: Set[str] = set()
        self._multiline_regex_supported_file_types: Set[str] = set()
        detectors = load_detectors()

        for detector in detectors:
            try:
                if detector.get("isMultiline"):
                    self.multiline_deny_list.add(re.compile('{}'.format(detector["Regex"])))
                    self.multiline_regex_to_metadata[detector["Regex"]] = detector
                    continue
                self.denylist.add(re.compile('{}'.format(detector["Regex"])))
                self.regex_to_metadata[detector["Regex"]] = detector
            except Exception:
                logging.error(
                    f"Failed to load detector {detector.get('Name')} with regex {detector.get('Regex')}",
                    exc_info=True,
                )

    @property
    def multiline_regex_supported_file_types(self) -> Set[str]:
        if self._multiline_regex_supported_file_types:
            return self._multiline_regex_supported_file_types
        for regex in self.multiline_regex_to_metadata.values():
            self._multiline_regex_supported_file_types.update(regex.get("supportedFiles", []))
        return self._multiline_regex_supported_file_types

    def analyze_line(
            self,
            filename: str,
            line: str,
            line_number: int = 0,
            context: Optional[CodeSnippet] = None,
            raw_context: Optional[CodeSnippet] = None,
            **kwargs: Any
    ) -> Set[PotentialSecret]:
        """This examines a line and finds all possible secret values in it"""
        output: Set[PotentialSecret] = set()

        line_length = len(line)
        if line_length > CustomRegexDetector.MAX_LINE_LENGTH:
            logging.info(f"File {filename} Line {line_number} has a length of {line_length}, which is higher than the max {CustomRegexDetector.MAX_LINE_LENGTH}")
            return output

        self._find_potential_secret(
            filename=filename,
            string_to_analyze=line,
            output=output,
            line_number=line_number,
            context=raw_context,
            is_multiline=False,
            **kwargs
        )

        # ToDo: Comment out once fix performence #  type: ignore
        if filename not in self._analyzed_files:
            self._analyzed_files.add(filename)
            # We only want to read file if: there is regex supporting it & file size is not over MAX_FILE_SIZE
            if not self.multiline_regex_to_metadata.values() or \
                    not self.multiline_regex_supported_file_types or \
                    not any([filename.endswith(str(file_type)) for file_type in self.multiline_regex_supported_file_types]) or \
                    not 0 < get_file_size_safe(filename) < CustomRegexDetector.MAX_FILE_SIZE:
                return output
            file_content = read_file_safe(filename)
            if not file_content:
                return output

            self._find_potential_secret(
                filename=filename,
                string_to_analyze=file_content,
                output=output,
                line_number=1,
                context=raw_context,
                is_multiline=True,
                **kwargs
            )

        return output

    def _find_potential_secret(
            self,
            filename: str,
            string_to_analyze: str,
            output: Set[PotentialSecret],
            line_number: int = 0,
            context: Optional[CodeSnippet] = None,
            is_multiline: bool = False,
            is_added: bool = False,
            is_removed: bool = False,
            **kwargs: Any
    ) -> None:
        current_denylist: Set[Pattern[str]] = set()
        if is_multiline:
            # We want the multiline regex to execute only if current file is supported by them
            for regex in self.multiline_deny_list:
                regex_supported_files = self.multiline_regex_to_metadata.get(regex.pattern, {}).get("supportedFiles", [])
                if regex_supported_files and any([filename.endswith(regex_supported_file) for regex_supported_file in regex_supported_files]):
                    current_denylist.add(regex)
        else:
            current_denylist = self.denylist

        current_regex_to_metadata: dict[str, dict[str, Any]] = self.multiline_regex_to_metadata if is_multiline else self.regex_to_metadata
        kwargs["regex_denylist"] = current_denylist
        for match, regex in self.analyze_string(string_to_analyze, **kwargs):
            try:
                verified_result = call_function_with_arguments(self.verify, secret=match, context=context)
                is_verified = True if verified_result == VerifiedResult.VERIFIED_TRUE else False
            except Exception:
                is_verified = False
            regex_data = current_regex_to_metadata[regex.pattern]
            # Wrap multiline match with fstring + ''
            match = f"'{match}'" if is_multiline else match
            ps = PotentialSecret(
                type=regex_data["Name"],
                filename=filename,
                secret=match,
                line_number=line_number,
                is_verified=is_verified,
                is_added=is_added,
                is_removed=is_removed,
                is_multiline=is_multiline,
            )
            ps.check_id = current_regex_to_metadata[regex.pattern]["Check_ID"]
            if is_multiline:
                output.add(ps)
            elif len(cast(str, ps.secret_value)) in range(MIN_CHARACTERS, MAX_CHARACTERS) or not regex_data['isCustom']:
                output.add(ps)
            else:
                logging.info(
                    f'Finding for check {ps.check_id} are not 5-100 characters in length, was ignored')

    def analyze_string(self, string: str, **kwargs: Optional[Dict[str, Any]]) -> Generator[Tuple[str, Pattern[str]], None, None]:  # type:ignore[override]
        regex_denylist: Set[Pattern[str]] = kwargs.get("regex_denylist", self.denylist)  # type: ignore[assignment]
        for regex in regex_denylist:
            for match in regex.findall(string):
                if isinstance(match, tuple):
                    for submatch in filter(bool, match):
                        # It might make sense to paste break after yielding
                        yield submatch, regex
                else:
                    yield match, regex



from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceValueCheck
from checkov.common.models.enums import CheckCategories


class APIGatewayMethodSettingCacheEncrypted(BaseResourceValueCheck):

    def __init__(self):
        """
        NIST.800-53.r5 CA-9(1), NIST.800-53.r5 CM-3(6), NIST.800-53.r5 SC-13, NIST.800-53.r5 SC-28,
        NIST.800-53.r5 SC-28(1), NIST.800-53.r5 SC-7(10), NIST.800-53.r5 SI-7(6)
        """
        name = "Ensure API Gateway method setting caching is set to encrypted"
        id = "CKV_AWS_308"
        supported_resources = ['aws_api_gateway_method_settings']
        categories = [CheckCategories.ENCRYPTION]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self):
        return "settings/[0]/cache_data_encrypted"


check = APIGatewayMethodSettingCacheEncrypted()



from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceValueCheck
from checkov.common.models.enums import CheckCategories


class APIGatewayMethodSettingCacheEnabled(BaseResourceValueCheck):

    def __init__(self):
        name = "Ensure API Gateway method setting caching is enabled"
        id = "CKV_AWS_225"
        supported_resources = ['aws_api_gateway_method_settings']
        categories = [CheckCategories.BACKUP_AND_RECOVERY]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self):
        return "settings/[0]/caching_enabled"


check = APIGatewayMethodSettingCacheEnabled()



import re
from typing import Any

from checkov.common.util.parser_utils import find_var_blocks

TF_OPERATOR_PREFIXES = ("lookup(", "list(", "file(")

CFN_VARIABLE_DEPENDANT_REGEX = re.compile(r"(?:Ref)\.\S+")
TF_BLOCK_REFS = ("var.", "local.", "module.", "data.")
TF_PROVIDER_PREFIXES = (
    "aws_",
    "azurerm_",
    "azuread_",
    "digitalocean_",
    "google_",
    "github_",
    "kubernetes_",
    "linode_",
    "oci_",
    "openstack_",
    "yandex_",
)


def is_terraform_variable_dependent(value: Any) -> bool:
    if not isinstance(value, str):
        return False

    if value.startswith(TF_BLOCK_REFS):
        return True

    if value.startswith(TF_PROVIDER_PREFIXES):
        return True

    if value.startswith(TF_OPERATOR_PREFIXES):
        return True

    if "${" not in value:
        return False

    if find_var_blocks(value):
        return True
    return False


def is_cloudformation_variable_dependent(value: Any) -> bool:
    if isinstance(value, str) and re.match(CFN_VARIABLE_DEPENDANT_REGEX, value):
        return True
    return False



from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.terraform.checks.resource.base_resource_check import BaseResourceCheck


class AKSDashboardDisabled(BaseResourceCheck):

    def __init__(self):
        name = "Ensure Kubernetes Dashboard is disabled"
        id = "CKV_AZURE_8"
        supported_resources = ['azurerm_kubernetes_cluster']
        categories = [CheckCategories.KUBERNETES]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf):
        addon_profile = conf.get('addon_profile')
        if addon_profile:
            addon_profile = addon_profile[0]
        self.evaluated_keys = ['addon_profile']
        if addon_profile and isinstance(addon_profile, dict):
            dashboard = addon_profile.get('kube_dashboard', [[]])[0]
            if isinstance(dashboard, dict):
                if dashboard.get('enabled')[0]:
                    self.evaluated_keys = ['addon_profile/kube_dashboard', 'addon_profile/kube_dashboard/[0]/enabled']
                    return CheckResult.FAILED
        return CheckResult.PASSED


check = AKSDashboardDisabled()



from __future__ import annotations

from typing import TYPE_CHECKING, Any

from checkov.common.util.data_structures_utils import find_in_dict
from checkov.kubernetes.image_referencer.base_provider import BaseKubernetesProvider

if TYPE_CHECKING:
    from networkx import DiGraph
    from checkov.common.images.graph.image_referencer_provider import _ExtractImagesCallableAlias


class KubernetesProvider(BaseKubernetesProvider):
    def __init__(self, graph_connector: DiGraph) -> None:
        super().__init__(
            graph_connector=graph_connector,
            supported_resource_types=SUPPORTED_K8S_IMAGE_RESOURCE_TYPES,
        )


def extract_images_from_cron_job(resource: dict[str, Any]) -> list[str]:
    spec = find_in_dict(input_dict=resource, key_path="spec/jobTemplate/spec/template/spec")
    return _extract_images_from_spec(spec)


def extract_images_from_pod(resource: dict[str, Any]) -> list[str]:
    spec = resource.get("spec")
    return _extract_images_from_spec(spec)


def extract_images_from_pod_template(resource: dict[str, Any]) -> list[str]:
    # the 'PodTemplate' object is usually not defined by the user, but rather used by Kubernetes internally
    spec = find_in_dict(input_dict=resource, key_path="template/spec")
    return _extract_images_from_spec(spec)


def extract_images_from_template(resource: dict[str, Any]) -> list[str]:
    spec = find_in_dict(input_dict=resource, key_path="spec/template/spec")
    return _extract_images_from_spec(spec)


def extract_images_from_containers(containers: Any) -> set[str]:
    """Helper function to extract image names from containers block"""

    image_names: set[str] = set()

    if isinstance(containers, list):
        for container in containers:
            if isinstance(container, dict):
                image = container.get("image")
                if image and isinstance(image, str):
                    image_names.add(image)

    return image_names


def _extract_images_from_spec(spec: dict[str, Any] | None) -> list[str]:
    image_names: set[str] = set()

    if isinstance(spec, dict):
        containers = spec.get("containers")
        image_names.update(extract_images_from_containers(containers=containers))

        containers = spec.get("initContainers")
        image_names.update(extract_images_from_containers(containers=containers))

    # Makes sure we return no duplications
    return list(image_names)


# needs to be at the bottom to add the defined functions
SUPPORTED_K8S_IMAGE_RESOURCE_TYPES: "dict[str, _ExtractImagesCallableAlias]" = {
    "CronJob": extract_images_from_cron_job,
    "Deployment": extract_images_from_template,
    "DeploymentConfig": extract_images_from_template,
    "DaemonSet": extract_images_from_template,
    "Job": extract_images_from_template,
    "Pod": extract_images_from_pod,
    "PodTemplate": extract_images_from_pod_template,
    "ReplicaSet": extract_images_from_template,
    "ReplicationController": extract_images_from_template,
    "StatefulSet": extract_images_from_template,
}



#!/usr/bin/env python
from __future__ import annotations

import atexit
import itertools
import json
import logging
import os
import shutil
import signal
import sys
import platform
from collections import defaultdict
from pathlib import Path
from typing import TYPE_CHECKING

import argcomplete
import configargparse
from urllib3.exceptions import MaxRetryError

import checkov.logging_init  # noqa  # should be imported before the others to ensure correct logging setup

from checkov.ansible.runner import Runner as ansible_runner
from checkov.argo_workflows.runner import Runner as argo_workflows_runner
from checkov.arm.runner import Runner as arm_runner
from checkov.azure_pipelines.runner import Runner as azure_pipelines_runner
from checkov.bitbucket.runner import Runner as bitbucket_configuration_runner
from checkov.bitbucket_pipelines.runner import Runner as bitbucket_pipelines_runner
from checkov.cloudformation.runner import Runner as cfn_runner
from checkov.common.bridgecrew.bc_source import SourceTypes, BCSourceType, get_source_type, SourceType
from checkov.common.bridgecrew.integration_features.features.policy_metadata_integration import \
    integration as policy_metadata_integration
from checkov.common.bridgecrew.integration_features.features.repo_config_integration import \
    integration as repo_config_integration
from checkov.common.bridgecrew.integration_features.features.suppressions_integration import \
    integration as suppressions_integration
from checkov.common.bridgecrew.integration_features.features.custom_policies_integration import \
    integration as custom_policies_integration
from checkov.common.bridgecrew.integration_features.integration_feature_registry import integration_feature_registry
from checkov.common.bridgecrew.platform_integration import bc_integration
from checkov.common.bridgecrew.integration_features.features.licensing_integration import integration as licensing_integration
from checkov.common.bridgecrew.severities import BcSeverities
from checkov.common.goget.github.get_git import GitGetter
from checkov.common.output.baseline import Baseline
from checkov.common.bridgecrew.check_type import checkov_runners, CheckType
from checkov.common.resource_code_logger_filter import add_resource_code_filter_to_logger
from checkov.common.runners.runner_registry import RunnerRegistry
from checkov.common.util import prompt
from checkov.common.util.banner import banner as checkov_banner, tool as checkov_tool
from checkov.common.util.config_utils import get_default_config_paths
from checkov.common.util.consts import CHECKOV_RUN_SCA_PACKAGE_SCAN_V2
from checkov.common.util.ext_argument_parser import ExtArgumentParser
from checkov.common.util.runner_dependency_handler import RunnerDependencyHandler
from checkov.common.util.type_forcers import convert_str_to_bool
from checkov.contributor_metrics import report_contributor_metrics
from checkov.dockerfile.runner import Runner as dockerfile_runner
from checkov.docs_generator import print_checks
from checkov.github.runner import Runner as github_configuration_runner
from checkov.github_actions.runner import Runner as github_actions_runner
from checkov.gitlab.runner import Runner as gitlab_configuration_runner
from checkov.gitlab_ci.runner import Runner as gitlab_ci_runner
from checkov.helm.runner import Runner as helm_runner
from checkov.json_doc.runner import Runner as json_runner
from checkov.kubernetes.runner import Runner as k8_runner
from checkov.kustomize.runner import Runner as kustomize_runner
from checkov.runner_filter import RunnerFilter
from checkov.sca_image.runner import Runner as sca_image_runner
from checkov.sca_package.runner import Runner as sca_package_runner
from checkov.sca_package_2.runner import Runner as sca_package_runner_2
from checkov.secrets.runner import Runner as secrets_runner
from checkov.serverless.runner import Runner as sls_runner
from checkov.terraform.plan_runner import Runner as tf_plan_runner
from checkov.terraform.runner import Runner as tf_graph_runner
from checkov.terraform_json.runner import TerraformJsonRunner
from checkov.version import version
from checkov.yaml_doc.runner import Runner as yaml_runner
from checkov.bicep.runner import Runner as bicep_runner
from checkov.openapi.runner import Runner as openapi_runner
from checkov.circleci_pipelines.runner import Runner as circleci_pipelines_runner
from checkov.logging_init import log_stream as logs_stream

if TYPE_CHECKING:
    from checkov.common.output.report import Report
    from configargparse import Namespace
    from typing_extensions import Literal
    from igraph import Graph
    from networkx import DiGraph

signal.signal(signal.SIGINT, lambda x, y: sys.exit(''))

outer_registry = None

logger = logging.getLogger(__name__)
add_resource_code_filter_to_logger(logger)

# sca package runner added during the run method
DEFAULT_RUNNERS = [
    tf_graph_runner(),
    cfn_runner(),
    k8_runner(),
    sls_runner(),
    arm_runner(),
    tf_plan_runner(),
    helm_runner(),
    dockerfile_runner(),
    secrets_runner(),
    json_runner(),
    yaml_runner(),
    github_configuration_runner(),
    gitlab_configuration_runner(),
    gitlab_ci_runner(),
    bitbucket_configuration_runner(),
    bitbucket_pipelines_runner(),
    kustomize_runner(),
    github_actions_runner(),
    bicep_runner(),
    openapi_runner(),
    sca_image_runner(),
    argo_workflows_runner(),
    circleci_pipelines_runner(),
    azure_pipelines_runner(),
    ansible_runner(),
    TerraformJsonRunner(),
]


class Checkov:
    def __init__(self, argv: list[str] = sys.argv[1:]) -> None:
        self.config: "Namespace"  # set in 'parse_config()'
        self.parser: "ExtArgumentParser"  # set in 'parse_config()'
        self.runners = DEFAULT_RUNNERS
        self.scan_reports: "list[Report]" = []
        self.run_metadata: dict[str, str | list[str]] = {}
        self.graphs: dict[str, DiGraph | Graph] = {}
        self.url: str | None = None

        self.parse_config(argv=argv)

    def _parse_mask_to_resource_attributes_to_omit(self) -> None:
        resource_attributes_to_omit = defaultdict(set)
        for entry in self.config.mask:
            splitted_entry = entry.split(':')
            # if we have 2 entries, this is resource & variable to mask
            splitted_entry_len = len(splitted_entry)
            if 2 == splitted_entry_len:
                resource = splitted_entry[0]
                variables_to_mask = splitted_entry[1].split(',')
                resource_attributes_to_omit[resource].update(variables_to_mask)
            # ToDo: Uncomment if we want to support universal masking
            # elif 1 == splitted_entry_len:
            #     variables_to_mask = splitted_entry[0].split(',')
            #     resource_attributes_to_omit[RESOURCE_ATTRIBUTES_TO_OMIT_UNIVERSAL_MASK].update(variables_to_mask)

        self.config.mask = resource_attributes_to_omit

    def parse_config(self, argv: list[str] = sys.argv[1:]) -> None:
        """Parses the user defined config via CLI flags"""

        default_config_paths = get_default_config_paths(sys.argv[1:])
        self.parser = ExtArgumentParser(
            description='Infrastructure as code static analysis',
            default_config_files=default_config_paths,
            config_file_parser_class=configargparse.YAMLConfigFileParser,
            add_env_var_help=True,
        )
        self.parser.add_parser_args()
        argcomplete.autocomplete(self.parser)

        self.config = self.parser.parse_args(argv)
        self.normalize_config()

    def normalize_config(self) -> None:
        if self.config.no_guide:
            logger.warning(
                '--no-guide is deprecated and will be removed in a future release. Use --skip-download instead'
            )
            self.config.skip_download = True
        if self.config.skip_suppressions:
            logger.warning(
                '--skip-suppressions is deprecated and will be removed in a future release. Use --skip-download instead'
            )
            self.config.skip_download = True
        if self.config.skip_policy_download:
            logger.warning(
                '--skip-policy-download is deprecated and will be removed in a future release. Use --skip-download instead'
            )
            self.config.skip_download = True

        elif not self.config.bc_api_key and not self.config.include_all_checkov_policies:
            # makes it easier to pick out policies later if we can just always rely on this flag without other context
            logger.debug('No API key present; setting include_all_checkov_policies to True')
            self.config.include_all_checkov_policies = True

        if self.config.use_enforcement_rules and not self.config.bc_api_key:
            self.parser.error('Must specify an API key with --use-enforcement-rules')

        if self.config.policy_metadata_filter and not (self.config.bc_api_key and self.config.prisma_api_url):
            logger.warning(
                '--policy-metadata-filter flag was used without a Prisma Cloud API key. Policy filtering will be skipped.'
            )

        # Parse mask into json with default dict. If self.config.mask is empty list, default dict will be assigned
        self._parse_mask_to_resource_attributes_to_omit()

        if self.config.file:
            # it is passed as a list of lists
            self.config.file = list(itertools.chain.from_iterable(self.config.file))

    def run(self, banner: str = checkov_banner, tool: str = checkov_tool, source_type: SourceType | None = None) -> int | None:
        self.run_metadata = {
            "checkov_version": version,
            "python_executable": sys.executable,
            "python_version": sys.version,
            "checkov_executable": sys.argv[0],
            "args": self.parser.format_values(sanitize=True).split('\n'),
            "OS_system_info": platform.platform(),
            "CPU_architecture": platform.processor(),
            "Python_implementation": platform.python_implementation()
        }

        logger.debug(f'Run metadata: {json.dumps(self.run_metadata, indent=2)}')
        try:
            if self.config.add_check:
                resp = prompt.Prompt()
                check = prompt.Check(resp.responses)
                check.action()
                return None

            # Check if --output value is None. If so, replace with ['cli'] for default cli output.
            if self.config.output is None:
                self.config.output = ['cli']

            if self.config.support:
                bc_integration.support_flag_enabled = True

            if self.config.bc_api_key and not self.config.include_all_checkov_policies:
                if self.config.skip_download and not self.config.external_checks_dir:
                    print('You are using an API key along with --skip-download but not --include-all-checkov-policies or --external-checks-dir. '
                          'With these arguments, Checkov cannot fetch metadata to determine what is a local Checkov-only '
                          'policy and what is a platform policy, so no policies will be evaluated. Please re-run Checkov '
                          'and either remove the --skip-download option, or use the --include-all-checkov-policies and / or '
                          '--external-checks-dir options.',
                          file=sys.stderr)
                    self.exit_run()
                elif self.config.skip_download:
                    print('You are using an API key along with --skip-download but not --include-all-checkov-policies. '
                          'With these arguments, Checkov cannot fetch metadata to determine what is a local Checkov-only '
                          'policy and what is a platform policy, so only local custom policies loaded with --external-checks-dir '
                          'will be evaluated.',
                          file=sys.stderr)
                else:
                    logger.debug('Using API key and not --include-all-checkov-policies - only running platform policies '
                                 '(this is the default behavior, and this message is just for debugging purposes)')

            # bridgecrew uses both the urllib3 and requests libraries, while checkov uses the requests library.
            # Allow the user to specify a CA bundle to be used by both libraries.
            bc_integration.setup_http_manager(self.config.ca_certificate, self.config.no_cert_verify)

            # if a repo is passed in it'll save it.  Otherwise a default will be created based on the file or dir
            self.config.repo_id = bc_integration.persist_repo_id(self.config)
            # if a bc_api_key is passed it'll save it.  Otherwise it will check ~/.bridgecrew/credentials
            self.config.bc_api_key = bc_integration.persist_bc_api_key(self.config)

            if not self.config.bc_api_key:
                # check, if someone tries to use a severity filter without an API key
                severities = {severity for severity in BcSeverities.__dict__.values() if isinstance(severity, str)}
                if (
                    (self.config.check and any(check in severities for check in self.config.check))
                    or (self.config.skip_check and any(check in severities for check in self.config.skip_check))
                ):
                    logging.warning("Filtering checks by severity is only possible with an API key")

            excluded_paths = self.config.skip_path or []

            if self.config.var_file:
                self.config.var_file = [os.path.abspath(f) for f in self.config.var_file]

            runner_filter = RunnerFilter(
                framework=self.config.framework,
                skip_framework=self.config.skip_framework,
                checks=self.config.check,
                skip_checks=self.config.skip_check,
                include_all_checkov_policies=self.config.include_all_checkov_policies,
                download_external_modules=bool(convert_str_to_bool(self.config.download_external_modules)),
                external_modules_download_path=self.config.external_modules_download_path,
                evaluate_variables=bool(convert_str_to_bool(self.config.evaluate_variables)),
                runners=checkov_runners,
                excluded_paths=excluded_paths,
                all_external=self.config.run_all_external_checks,
                var_files=self.config.var_file,
                skip_cve_package=self.config.skip_cve_package,
                show_progress_bar=not self.config.quiet,
                use_enforcement_rules=self.config.use_enforcement_rules,
                enable_secret_scan_all_files=bool(convert_str_to_bool(self.config.enable_secret_scan_all_files)),
                block_list_secret_scan=self.config.block_list_secret_scan,
                deep_analysis=self.config.deep_analysis,
                repo_root_for_plan_enrichment=self.config.repo_root_for_plan_enrichment,
                resource_attr_to_omit=self.config.mask,
                enable_git_history_secret_scan=self.config.scan_secrets_history,
                git_history_timeout=self.config.secrets_history_timeout
            )

            source_env_val = os.getenv('BC_SOURCE', 'cli')
            source = source_type if source_type else get_source_type(source_env_val)
            if source == SourceTypes[BCSourceType.DISABLED]:
                logger.warning(
                    f'Received unexpected value for BC_SOURCE: {source_env_val}; Should be one of {{{",".join(SourceTypes.keys())}}} setting source to DISABLED')
            source_version = os.getenv('BC_SOURCE_VERSION', version)
            logger.debug(f'BC_SOURCE = {source.name}, version = {source_version}')

            if self.config.list:
                # This speeds up execution by not setting up upload credentials (since we won't upload anything anyways)
                logger.debug('Using --list; setting source to DISABLED')
                source = SourceTypes[BCSourceType.DISABLED]

            if CHECKOV_RUN_SCA_PACKAGE_SCAN_V2:
                self.runners.append(sca_package_runner_2())
            else:
                self.runners.append(sca_package_runner())

            if outer_registry:
                runner_registry = outer_registry
                runner_registry.runner_filter = runner_filter
                runner_registry.filter_runner_framework()
            else:
                runner_registry = RunnerRegistry(banner, runner_filter, *self.runners, tool=tool)

            runnerDependencyHandler = RunnerDependencyHandler(runner_registry)
            runnerDependencyHandler.validate_runner_deps()

            if self.config.show_config:
                print(self.parser.format_values(sanitize=True))
                return None

            if self.config.bc_api_key == '':
                self.parser.error(
                    'The --bc-api-key flag was specified but the value was blank. If this value was passed as a '
                    'secret, you may need to double check the mapping.'
                )
            elif self.config.bc_api_key:
                logger.debug(f'Using API key ending with {self.config.bc_api_key[-8:]}')

                if not bc_integration.is_token_valid(self.config.bc_api_key):
                    raise Exception('The provided API key does not appear to be a valid Bridgecrew API key or Prisma Cloud '
                                    'access key and secret key. For Prisma, the value must be in the form '
                                    'ACCESS_KEY::SECRET_KEY. For Bridgecrew, make sure to copy the token value from when you '
                                    'created it, not the token ID visible later on. If you are using environment variables, '
                                    'make sure they are properly set and exported.')

                if not self.config.list:
                    # if you are only listing policies, then the API key will be used to fetch policies, but that's it,
                    # so the repo is not required and ignored
                    if self.config.repo_id is None:
                        self.parser.error("--repo-id argument is required when using --bc-api-key")
                    else:
                        repo_id_sections = self.config.repo_id.split('/')
                        if len(repo_id_sections) < 2 or any(len(section) == 0 for section in repo_id_sections):
                            self.parser.error(
                                "--repo-id argument format should be 'organization/repository_name' E.g "
                                "bridgecrewio/checkov"
                            )

                try:
                    bc_integration.bc_api_key = self.config.bc_api_key
                    bc_integration.setup_bridgecrew_credentials(repo_id=self.config.repo_id,
                                                                skip_fixes=self.config.skip_fixes,
                                                                skip_download=self.config.skip_download,
                                                                source=source,
                                                                source_version=source_version,
                                                                repo_branch=self.config.branch,
                                                                prisma_api_url=self.config.prisma_api_url)

                except MaxRetryError:
                    self.exit_run()
                except Exception:
                    if bc_integration.prisma_api_url:
                        message = 'An error occurred setting up the Prisma Cloud platform integration. ' \
                                  'Please check your Prisma Cloud API token and URL and try again.'
                    else:
                        message = 'An error occurred setting up the Bridgecrew platform integration. ' \
                                  'Please check your API token and try again.'
                    if logger.isEnabledFor(logging.DEBUG):
                        logger.debug(message, exc_info=True)
                    else:
                        logger.error(message)
                        logger.error('Please try setting the environment variable LOG_LEVEL=DEBUG and re-running the command, and provide the output to support', exc_info=True)
                    self.exit_run()
            else:
                if self.config.support:
                    logger.warning("--bc-api-key argument is required when using --support")
                logger.debug('No API key found. Scanning locally only.')
                self.config.include_all_checkov_policies = True

            if self.config.check and self.config.skip_check:
                if any(item in runner_filter.checks for item in runner_filter.skip_checks):
                    self.parser.error("The check ids specified for '--check' and '--skip-check' must be mutually exclusive.")
                    return None

            BC_SKIP_MAPPING = os.getenv("BC_SKIP_MAPPING", "FALSE")
            if self.config.skip_download or BC_SKIP_MAPPING.upper() == "TRUE":
                bc_integration.skip_download = True

            try:
                bc_integration.get_platform_run_config()
            except Exception:
                if not self.config.include_all_checkov_policies:
                    # stack trace gets printed in the exception handlers above
                    # include_all_checkov_policies will always be set when there is no API key, so we don't need to worry about it here
                    print('An error occurred getting data from the platform, including policy metadata. Because --include-all-checkov-policies '
                          'was not used, Checkov cannot differentiate Checkov-only policies from platform policies, and no '
                          'policies will get evaluated. Please resolve the error above or re-run with the --include-all-checkov-policies argument '
                          '(but note that this will not include any custom platform configurations or policy metadata).',
                          file=sys.stderr)
                    self.exit_run()

            bc_integration.get_prisma_build_policies(self.config.policy_metadata_filter)

            # set config to make it usable inside the integration features
            integration_feature_registry.config = self.config
            integration_feature_registry.run_pre_scan()

            policy_level_suppression = suppressions_integration.get_policy_level_suppressions()
            bc_cloned_checks = custom_policies_integration.bc_cloned_checks
            runner_filter.bc_cloned_checks = bc_cloned_checks
            custom_policies_integration.policy_level_suppression = list(policy_level_suppression.keys())

            if any(framework in runner_filter.framework for framework in ("all", CheckType.SCA_IMAGE)):
                # only run image referencer, when sca_image framework is enabled
                runner_filter.run_image_referencer = licensing_integration.should_run_image_referencer()

            runner_filter.filtered_policy_ids = policy_metadata_integration.filtered_policy_ids
            logger.debug(f"Filtered list of policies: {runner_filter.filtered_policy_ids}")

            runner_filter.excluded_paths = runner_filter.excluded_paths + list(repo_config_integration.skip_paths)
            policy_level_suppression = suppressions_integration.get_policy_level_suppressions()
            bc_cloned_checks = custom_policies_integration.bc_cloned_checks
            runner_filter.bc_cloned_checks = bc_cloned_checks
            custom_policies_integration.policy_level_suppression = list(policy_level_suppression.keys())
            runner_filter.set_suppressed_policies(list(policy_level_suppression.values()))

            if self.config.use_enforcement_rules:
                runner_filter.apply_enforcement_rules(repo_config_integration.code_category_configs)

            if self.config.list:
                print_checks(frameworks=self.config.framework, use_bc_ids=self.config.output_bc_ids,
                             include_all_checkov_policies=self.config.include_all_checkov_policies,
                             filtered_policy_ids=runner_filter.filtered_policy_ids)
                return None

            baseline = None
            if self.config.baseline:
                baseline = Baseline(self.config.output_baseline_as_skipped)
                baseline.from_json(self.config.baseline)

            external_checks_dir = self.get_external_checks_dir()
            created_baseline_path = None

            default_github_dir_path = os.getcwd() + '/' + os.getenv('CKV_GITHUB_CONF_DIR_NAME', 'github_conf')
            git_configuration_folders = [os.getenv("CKV_GITHUB_CONF_DIR_PATH", default_github_dir_path),
                                         os.getcwd() + '/' + os.getenv('CKV_GITLAB_CONF_DIR_NAME', 'gitlab_conf')]

            if self.config.directory:
                exit_codes = []
                for root_folder in self.config.directory:
                    absolute_root_folder = os.path.abspath(root_folder)
                    if not os.path.exists(root_folder):
                        logger.error(f'Directory {root_folder} does not exist; skipping it')
                        continue
                    file = self.config.file
                    self.scan_reports = runner_registry.run(
                        root_folder=root_folder,
                        external_checks_dir=external_checks_dir,
                        files=file,
                    )
                    self.graphs = runner_registry.check_type_to_graph
                    if runner_registry.is_error_in_reports(self.scan_reports):
                        self.exit_run()
                    if baseline:
                        baseline.compare_and_reduce_reports(self.scan_reports)

                    if bc_integration.is_integration_configured() \
                            and bc_integration.bc_source \
                            and bc_integration.bc_source.upload_results \
                            and not self.config.skip_results_upload:
                        included_paths = [self.config.external_modules_download_path]
                        for r in runner_registry.runners:
                            included_paths.extend(r.included_paths())
                        self.upload_results(
                            root_folder=root_folder,
                            absolute_root_folder=absolute_root_folder,
                            excluded_paths=runner_filter.excluded_paths,
                            included_paths=included_paths,
                            git_configuration_folders=git_configuration_folders,
                        )

                    if self.config.create_baseline:
                        overall_baseline = Baseline()
                        for report in self.scan_reports:
                            overall_baseline.add_findings_from_report(report)
                        created_baseline_path = os.path.join(os.path.abspath(root_folder), '.checkov.baseline')
                        with open(created_baseline_path, 'w') as f:
                            json.dump(overall_baseline.to_dict(), f, indent=4)
                    exit_codes.append(self.print_results(
                        runner_registry=runner_registry,
                        url=self.url,
                        created_baseline_path=created_baseline_path,
                        baseline=baseline,
                    ))

                # this needs to run after the upload (otherwise the repository does not exist)
                should_run_contributor_metrics = bc_integration.bc_api_key and self.config.repo_id and self.config.prisma_api_url
                logger.debug(f"Should run contributor metrics report: {should_run_contributor_metrics}")
                if should_run_contributor_metrics:
                    try:  # collect contributor info and upload
                        report_contributor_metrics(self.config.repo_id, source.name, bc_integration)
                    except Exception as e:
                        logger.warning(f"Unable to report contributor metrics due to: {e}")

                exit_code = 1 if 1 in exit_codes else 0
                return exit_code
            elif self.config.docker_image:
                if self.config.bc_api_key is None:
                    self.parser.error("--bc-api-key argument is required when using --docker-image")
                    return None
                if self.config.dockerfile_path is None:
                    self.parser.error("--dockerfile-path argument is required when using --docker-image")
                    return None
                if self.config.branch is None:
                    self.parser.error("--branch argument is required when using --docker-image")
                    return None
                files = [os.path.abspath(self.config.dockerfile_path)]
                runner = sca_image_runner()
                result = runner.run(
                    root_folder='',
                    image_id=self.config.docker_image,
                    dockerfile_path=self.config.dockerfile_path,
                    runner_filter=runner_filter,
                )
                self.scan_reports = result if isinstance(result, list) else [result]
                if runner_registry.is_error_in_reports(self.scan_reports):
                    self.exit_run()
                if len(self.scan_reports) > 1:
                    # this shouldn't happen, but if it happens, then it is intended or something is broke
                    logger.error(f"SCA image runner returned {len(self.scan_reports)} reports; expected 1")

                integration_feature_registry.run_post_runner(self.scan_reports[0])

                if not self.config.skip_results_upload:
                    bc_integration.persist_repository(os.path.dirname(self.config.dockerfile_path), files=files)
                    bc_integration.persist_scan_results(self.scan_reports)
                    bc_integration.persist_image_scan_results(runner.raw_report, self.config.dockerfile_path,
                                                              self.config.docker_image,
                                                              self.config.branch)

                    bc_integration.persist_run_metadata(self.run_metadata)
                    if bc_integration.enable_persist_graphs:
                        bc_integration.persist_graphs(self.graphs)
                    self.url = self.commit_repository()

                should_run_contributor_metrics = bc_integration.bc_api_key and self.config.repo_id and self.config.prisma_api_url
                logger.debug(f"Should run contributor metrics report: {should_run_contributor_metrics}")
                if should_run_contributor_metrics:
                    try:  # collect contributor info and upload
                        report_contributor_metrics(self.config.repo_id, source.name, bc_integration)
                    except Exception as e:
                        logger.warning(f"Unable to report contributor metrics due to: {e}")

                exit_code = self.print_results(runner_registry=runner_registry, url=self.url)
                return exit_code
            elif self.config.file:
                runner_registry.filter_runners_for_files(self.config.file)
                self.scan_reports = runner_registry.run(
                    external_checks_dir=external_checks_dir,
                    files=self.config.file,
                    repo_root_for_plan_enrichment=self.config.repo_root_for_plan_enrichment,
                )
                self.graphs = runner_registry.check_type_to_graph
                if runner_registry.is_error_in_reports(self.scan_reports):
                    self.exit_run()
                if baseline:
                    baseline.compare_and_reduce_reports(self.scan_reports)
                if self.config.create_baseline:
                    overall_baseline = Baseline()
                    for report in self.scan_reports:
                        overall_baseline.add_findings_from_report(report)
                    created_baseline_path = os.path.join(os.path.abspath(os.path.commonprefix(self.config.file)),
                                                         '.checkov.baseline')
                    with open(created_baseline_path, 'w') as f:
                        json.dump(overall_baseline.to_dict(), f, indent=4)

                if bc_integration.is_integration_configured() \
                        and bc_integration.bc_source \
                        and bc_integration.bc_source.upload_results \
                        and not self.config.skip_results_upload:
                    files = [os.path.abspath(file) for file in self.config.file]
                    root_folder = os.path.split(os.path.commonprefix(files))[0]
                    absolute_root_folder = os.path.abspath(root_folder)

                    self.upload_results(
                        root_folder=root_folder,
                        absolute_root_folder=absolute_root_folder,
                        files=files,
                        excluded_paths=runner_filter.excluded_paths,
                        git_configuration_folders=git_configuration_folders,
                    )

                should_run_contributor_metrics = bc_integration.bc_api_key and self.config.repo_id and self.config.prisma_api_url
                logger.debug(f"Should run contributor metrics report: {should_run_contributor_metrics}")
                if should_run_contributor_metrics:
                    try:  # collect contributor info and upload
                        report_contributor_metrics(self.config.repo_id, source.name, bc_integration)
                    except Exception as e:
                        logger.warning(f"Unable to report contributor metrics due to: {e}")

                exit_code = self.print_results(
                    runner_registry=runner_registry,
                    url=self.url,
                    created_baseline_path=created_baseline_path,
                    baseline=baseline,
                )
                return exit_code
            elif not self.config.quiet:
                print(f"{banner}")

                bc_integration.onboarding()
            return None
        except BaseException:
            logging.error("Exception traceback:", exc_info=True)
            raise

        finally:
            if self.config.support:
                bc_integration.persist_logs_stream(logs_stream)

    def exit_run(self) -> None:
        exit(0) if self.config.no_fail_on_crash else exit(2)

    def commit_repository(self) -> str | None:
        try:
            return bc_integration.commit_repository(self.config.branch)
        except Exception:
            logging.debug("commit_repository failed, exiting", exc_info=True)
            self.exit_run()
            return ""

    def get_external_checks_dir(self) -> list[str]:
        external_checks_dir: "list[str]" = self.config.external_checks_dir
        if self.config.external_checks_git:
            git_getter = GitGetter(url=self.config.external_checks_git[0])
            external_checks_dir = [git_getter.get()]
            atexit.register(shutil.rmtree, str(Path(external_checks_dir[0]).parent))
        return external_checks_dir

    def upload_results(
            self,
            root_folder: str,
            absolute_root_folder: str,
            files: list[str] | None = None,
            excluded_paths: list[str] | None = None,
            included_paths: list[str] | None = None,
            git_configuration_folders: list[str] | None = None,
    ) -> None:
        """Upload scan results and other relevant files"""

        bc_integration.persist_repository(
            root_dir=root_folder,
            files=files,
            excluded_paths=excluded_paths,
            included_paths=included_paths,
        )
        if git_configuration_folders:
            bc_integration.persist_git_configuration(os.getcwd(), git_configuration_folders)
        bc_integration.persist_scan_results(self.scan_reports)
        bc_integration.persist_run_metadata(self.run_metadata)
        if bc_integration.enable_persist_graphs:
            bc_integration.persist_graphs(self.graphs, absolute_root_folder=absolute_root_folder)
        self.url = self.commit_repository()

    def print_results(
            self,
            runner_registry: RunnerRegistry,
            url: str | None = None,
            created_baseline_path: str | None = None,
            baseline: Baseline | None = None,
    ) -> Literal[0, 1]:
        """Print scan results to stdout"""

        if convert_str_to_bool(os.getenv("CHECKOV_NO_OUTPUT", "False")):
            # this is mainly used for testing, where the report output is not needed
            return 0

        return runner_registry.print_reports(
            scan_reports=self.scan_reports,
            config=self.config,
            url=url,
            created_baseline_path=created_baseline_path,
            baseline=baseline,
        )


# the flag/arg parsing moved to checkov/common/util/ext_argument_parser.py


if __name__ == '__main__':
    ckv = Checkov()
    sys.exit(ckv.run())



from typing import Dict, List, Union, Any
from checkov.common.graph.graph_builder import EncryptionTypes
from checkov.common.graph.graph_builder.graph_components.generic_resource_encryption_base import (
    GenericResourceEncryptionBase,
)
from checkov.common.util.data_structures_utils import get_empty_list_str


class GenericResourceEncryption(GenericResourceEncryptionBase):
    def __init__(
        self,
        resource_type: str,
        attribute_values_map: Dict[str, Union[List[bool], List[str]]],
        enabled_by_default: bool = False,
    ) -> None:
        super().__init__(
            resource_type, attribute_values_map, enabled_by_default, node_to_node_encryption="node_to_node_encryption"
        )
        if self.resource_type.startswith("aws_"):
            if self.resource_type == "aws_s3_bucket":
                # for s3 buckets the default is SSE-S3 which uses AES256
                self.default_description = EncryptionTypes.AES256.value
            else:
                self.default_description = EncryptionTypes.DEFAULT_KMS.value


# This map allows dynamically creating the check for each resource type based on GenericResourceEncryption.
# Please check out the constructor to understand all the edge cases.
ENCRYPTION_BY_RESOURCE_TYPE: Dict[str, Any] = {
    "aws_ecr_repository": GenericResourceEncryption(
        "aws_ecr_repository",
        {
            "encryption_configuration.encryption_type": [EncryptionTypes.AES256.value, EncryptionTypes.KMS_VALUE.value],
            "encryption_configuration.kms_key": get_empty_list_str(),
        },
    ),
    "aws_neptune_cluster": GenericResourceEncryption(
        "aws_neptune_cluster", {"storage_encrypted": [True], "kms_key_arn": get_empty_list_str()}
    ),
    "aws_efs_file_system": GenericResourceEncryption(
        "aws_efs_file_system", {"encrypted": [True], "kms_key_id": get_empty_list_str()}
    ),
    "aws_sagemaker_feature_group": GenericResourceEncryption(
        "aws_sagemaker_feature_group", {"security_config.kms_key_id": get_empty_list_str()}
    ),
    "aws_ebs_volume": GenericResourceEncryption(
        "aws_ebs_volume", {"encrypted": [True], "kms_key_id": get_empty_list_str()}
    ),
    "aws_elasticache_replication_group": GenericResourceEncryption(
        "aws_elasticache_replication_group", {"at_rest_encryption_enabled": [True], "kms_key_id": ["arn"]}
    ),
    "aws_elasticsearch_domain": GenericResourceEncryption(
        "aws_elasticsearch_domain",
        {
            "encrypt_at_rest.enabled": [True],
            "kms_key_id": get_empty_list_str(),
            "node_to_node_encryption.enabled": [True],
        },
    ),
    "aws_msk_cluster": GenericResourceEncryption(
        "aws_msk_cluster", {"encryption_info.encryption_at_rest_kms_key_arn": get_empty_list_str()}
    ),
    "aws_docdb_cluster": GenericResourceEncryption(
        "aws_docdb_cluster", {"storage_encrypted": [True], "kms_key_arn": get_empty_list_str()}
    ),
    "aws_codebuild_project": GenericResourceEncryption(
        "aws_codebuild_project", {"encryption_key": get_empty_list_str()}
    ),
    "aws_codebuild_report_group": GenericResourceEncryption(
        "aws_codebuild_report_group",
        {
            "export_config.s3_destination.encryption_disabled": [False],
            "export_config.s3_destination.encryption_key": get_empty_list_str(),
        },
    ),
    "aws_athena_database": GenericResourceEncryption(
        "aws_athena_database",
        {
            "encryption_configuration.encryption_option": ["SSE_S3", "SSE_KMS", "CSE_KMS"],
            "encryption_configuration.kms_key": get_empty_list_str(),
        },
    ),
    "aws_athena_workgroup": GenericResourceEncryption(
        "aws_athena_workgroup",
        {
            "configuration.result_configuration.encryption_configuration.encryption_option": [
                "SSE_S3",
                "SSE_KMS",
                "CSE_KMS",
            ],
            "configuration.result_configuration.encryption_configuration.kms_key_arn": get_empty_list_str(),
        },
    ),
    "aws_kinesis_stream": GenericResourceEncryption(
        "aws_kinesis_stream", {"encryption_type": [EncryptionTypes.KMS_VALUE.value], "kms_key_id": get_empty_list_str()}
    ),
    "aws_eks_cluster": GenericResourceEncryption(
        "aws_eks_cluster", {"encryption_config.provider.key_arn": get_empty_list_str()}
    ),
    "aws_dynamodb_table": GenericResourceEncryption(
        "aws_dynamodb_table",
        {"server_side_encryption.enabled": [True], "server_side_encryption.kms_key_arn": get_empty_list_str()},
        enabled_by_default=True,
    ),
    "aws_rds_cluster": GenericResourceEncryption(
        "aws_rds_cluster", {"storage_encrypted": [True], "kms_key_id": get_empty_list_str()}
    ),
    "aws_rds_global_cluster": GenericResourceEncryption("aws_rds_global_cluster", {"storage_encrypted": [True]}),
    "aws_s3_bucket": GenericResourceEncryption(
        "aws_s3_bucket",
        {
            "server_side_encryption_configuration.rule.apply_server_side_encryption_by_default.sse_algorithm": [
                EncryptionTypes.AWS_KMS_VALUE.value,
                EncryptionTypes.AES256.value,
            ],
            "server_side_encryption_configuration.rule.apply_server_side_encryption_by_default.kms_master_key_id": get_empty_list_str(),
        },
        enabled_by_default=True,
    ),
    "aws_s3_bucket_inventory": GenericResourceEncryption(
        "aws_s3_bucket_inventory", {"destination.bucket.encryption": get_empty_list_str()}
    ),
    "aws_s3_bucket_object": GenericResourceEncryption(
        "aws_s3_bucket_object",
        {
            "server_side_encryption": [EncryptionTypes.AWS_KMS_VALUE.value, EncryptionTypes.AES256.value],
            "kms_key_id": get_empty_list_str(),
        },
    ),
    "aws_cloudwatch_log_group": GenericResourceEncryption(
        "aws_cloudwatch_log_group", {"kms_key_id": get_empty_list_str()}, enabled_by_default=True
    ),
    "aws_cloudtrail": GenericResourceEncryption("aws_cloudtrail", {"kms_key_id": get_empty_list_str()}),
    "aws_dax_cluster": GenericResourceEncryption("aws_dax_cluster", {"server_side_encryption.enabled": [True]}),
    "aws_redshift_cluster": GenericResourceEncryption(
        "aws_redshift_cluster", {"encrypted": [True], "kms_key_id": get_empty_list_str()}
    ),
    "aws_sns_topic": GenericResourceEncryption("aws_sns_topic", {"kms_master_key_id": get_empty_list_str()}),
    "aws_sqs_queue": GenericResourceEncryption("aws_sqs_queue", {"kms_master_key_id": get_empty_list_str()}),
}



from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceCheck
from checkov.common.models.enums import CheckResult, CheckCategories


class NetworkACLUnrestricted(BaseResourceCheck):
    def __init__(self) -> None:
        name = "Ensure NACL ingress does not allow all Ports"
        id = "CKV_AWS_352"
        supported_resources = ('aws_network_acl_rule',)
        categories = (CheckCategories.NETWORKING,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf):
        egress = conf.get('egress')
        if egress and isinstance(egress, list) and egress[0]:
            return CheckResult.UNKNOWN
        port = conf.get('from_port')
        if port and isinstance(port, list) and port[0]:
            return CheckResult.PASSED
        return CheckResult.FAILED


check = NetworkACLUnrestricted()



from typing import Dict, Any

from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.terraform.checks.resource.base_resource_check import BaseResourceCheck


class CloudStorageSelfLogging(BaseResourceCheck):
    def __init__(self) -> None:
        name = "Bucket should not log to itself"
        id = "CKV_GCP_63"
        supported_resources = ["google_storage_bucket"]
        categories = [CheckCategories.LOGGING]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf: Dict[str, Any]) -> CheckResult:
        bucket_name = conf.get("name")
        # check for logging
        if "logging" in conf:
            self.evaluated_keys = ["logging"]
            if conf["logging"][0]:
                log_bucket_name = conf["logging"][0]["log_bucket"]
                self.evaluated_keys = ["logging/[0]/log_bucket", "name"]
                if log_bucket_name != bucket_name:
                    return CheckResult.PASSED
                return CheckResult.FAILED
            return CheckResult.FAILED
        return CheckResult.UNKNOWN


check = CloudStorageSelfLogging()



from __future__ import annotations

from typing import Any

from checkov.common.models.enums import CheckResult
from checkov.gitlab_ci.checks.base_gitlab_ci_check import BaseGitlabCICheck
from checkov.yaml_doc.enums import BlockType


class DetectImageUsage(BaseGitlabCICheck):
    def __init__(self) -> None:
        name = "Detecting image usages in gitlab workflows"
        id = "CKV_GITLABCI_3"
        super().__init__(
            name=name,
            id=id,
            block_type=BlockType.ARRAY,
            supported_entities=('*.image[]', '*.services[]')
        )

    def scan_conf(self, conf: dict[str, Any]) -> tuple[CheckResult, dict[str, Any]]:
        return CheckResult.PASSED, conf


check = DetectImageUsage()



from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.cloudformation.checks.resource.base_resource_value_check import BaseResourceValueCheck


class KMSRotation(BaseResourceValueCheck):
    def __init__(self) -> None:
        name = "Ensure rotation for customer created CMKs is enabled"
        id = "CKV_AWS_7"
        supported_resources = ("AWS::KMS::Key",)
        categories = (CheckCategories.ENCRYPTION,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self) -> str:
        return "Properties/EnableKeyRotation"

    def scan_resource_conf(self, conf):
        # Only symmetric keys support auto rotation. The attribute is optional and defaults to symmetric.
        properties = conf.get("Properties")
        if properties and isinstance(properties, dict):
            spec = properties.get("KeySpec")
            if spec and isinstance(spec, str):
                if 'SYMMETRIC_DEFAULT' not in spec and 'HMAC' not in spec:
                    return CheckResult.UNKNOWN
        return super().scan_resource_conf(conf)


check = KMSRotation()



from enum import Enum


class ResourceType(str, Enum):
    JOBS = "jobs"
    PERMISSIONS = "permissions"
    STEPS = "steps"
    ON = "on"

    def __str__(self) -> str:
        # needed, because of a Python 3.11 change
        return self.value



from __future__ import annotations

import re

from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.common.util.type_forcers import extract_policy_dict
from checkov.terraform.checks.resource.base_resource_check import BaseResourceCheck
from typing import Any

ACCOUNT_ACCESS = re.compile(r"\d{12}|arn:aws:iam::\d{12}:root")


class IAMRoleAllowAssumeFromAccount(BaseResourceCheck):
    def __init__(self) -> None:
        name = "Ensure AWS IAM policy does not allow assume role permission across all services"
        id = "CKV_AWS_61"
        supported_resources = ("aws_iam_role",)
        categories = (CheckCategories.IAM,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf: dict[str, list[Any]]) -> CheckResult:
        try:
            assume_role_block = extract_policy_dict(conf["assume_role_policy"][0])
            if assume_role_block and "Statement" in assume_role_block:
                for statement in assume_role_block["Statement"]:
                    if statement.get("Effect") == "Deny":
                        continue
                    if "Principal" in statement and "AWS" in statement["Principal"]:
                        # Can be a string or an array of strings
                        aws_principals = statement["Principal"]["AWS"]
                        if isinstance(aws_principals, str) and re.match(ACCOUNT_ACCESS, aws_principals):
                            return CheckResult.FAILED
                        elif isinstance(aws_principals, list):
                            for aws_principal in aws_principals:
                                if isinstance(aws_principal, str) and re.match(ACCOUNT_ACCESS, aws_principal):
                                    return CheckResult.FAILED
        except Exception:  # nosec
            return CheckResult.UNKNOWN

        return CheckResult.PASSED

    def get_evaluated_keys(self) -> list[str]:
        return ["assume_role_policy"]


check = IAMRoleAllowAssumeFromAccount()



from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceValueCheck


class ElasticCacheAutomaticMinorUpgrades(BaseResourceValueCheck):
    def __init__(self):
        """
        NIST.800-53.r5 SI-2, NIST.800-53.r5 SI-2(2), NIST.800-53.r5 SI-2(4), NIST.800-53.r5 SI-2(5)
        ElastiCache for Redis cache clusters should have auto minor version upgrades enabled
        """
        name = "Ensure ElastiCache for Redis cache clusters have auto minor version upgrades enabled"
        id = "CKV_AWS_322"
        supported_resources = ["aws_elasticache_cluster"]
        categories = [CheckCategories.GENERAL_SECURITY]
        super().__init__(
            name=name,
            id=id,
            categories=categories,
            supported_resources=supported_resources,
            missing_block_result=CheckResult.PASSED
        )

    def scan_resource_conf(self, conf):
        if conf.get("engine") == ["memcached"]:
            return CheckResult.UNKNOWN

        return super().scan_resource_conf(conf)

    def get_inspected_key(self):
        return "auto_minor_version_upgrade"


check = ElasticCacheAutomaticMinorUpgrades()



from __future__ import annotations

from checkov.common.models.enums import CheckCategories
from checkov.terraform.checks.resource.base_resource_negative_value_check import BaseResourceNegativeValueCheck
from typing import Any


class LambdaFunctionIsNotPublic(BaseResourceNegativeValueCheck):
    def __init__(self) -> None:
        """
        PCI DSS v3.2.1/1.2.1, PCI DSS v3.2.1/1.3.1, PCI DSS v3.2.1/1.3.2, PCI DSS v3.2.1/1.3.4, PCI DSS v3.2.1/7.2.1,
        NIST.800-53.r5 AC-21, NIST.800-53.r5 AC-3, NIST.800-53.r5 AC-3(7), NIST.800-53.r5 AC-4, NIST.800-53.r5 AC-4(21),
        NIST.800-53.r5 AC-6, NIST.800-53.r5 SC-7, NIST.800-53.r5 SC-7(11), NIST.800-53.r5 SC-7(16), NIST.800-53.r5 SC-7(20),
        NIST.800-53.r5 SC-7(21), NIST.800-53.r5 SC-7(3), NIST.800-53.r5 SC-7(4), NIST.800-53.r5 SC-7(9)
        Lambda function policies should prohibit public access
        """

        name = "Ensure that AWS Lambda function is not publicly accessible"
        id = "CKV_AWS_301"
        supported_resources = ("aws_lambda_permission",)
        categories = (CheckCategories.GENERAL_SECURITY,)
        super().__init__(
            name=name,
            id=id,
            categories=categories,
            supported_resources=supported_resources,
        )

    def get_inspected_key(self) -> str:
        return "principal"

    def get_forbidden_values(self) -> list[Any]:
        return ["*"]


check = LambdaFunctionIsNotPublic()



from __future__ import annotations

from typing import Any

from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.common.util.type_forcers import force_int
from checkov.terraform.checks.resource.base_resource_check import BaseResourceCheck


class CloudFrontResponseHeaderStrictTransportSecurity(BaseResourceCheck):
    def __init__(self) -> None:
        name = "Ensure CloudFront response header policy enforces Strict Transport Security"
        id = "CKV_AWS_259"
        supported_resources = ("aws_cloudfront_response_headers_policy",)
        categories = (CheckCategories.GENERAL_SECURITY,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf: dict[str, list[Any]]) -> CheckResult:
        # https://developer.mozilla.org/en-US/docs/Web/HTTP/Headers/Strict-Transport-Security

        self.evaluated_keys = ["security_headers_config"]
        security_headers = conf.get("security_headers_config")
        if security_headers and isinstance(security_headers, list):
            self.evaluated_keys = ["security_headers_config/[0]/strict_transport_security"]
            sts = security_headers[0].get("strict_transport_security")
            if sts and isinstance(sts, list):
                # if one of those configs is not set correctly, then the check should fail
                self.evaluated_keys = [
                    "security_headers_config/[0]/strict_transport_security/[0]/access_control_max_age_sec"
                ]
                max_age = sts[0].get("access_control_max_age_sec")
                if not max_age:
                    return CheckResult.FAILED
                else:
                    max_age_int = force_int(max_age[0])
                    if not max_age_int or max_age_int < 31536000:  # 1 year
                        return CheckResult.FAILED

                self.evaluated_keys.append(
                    "security_headers_config/[0]/strict_transport_security/[0]/include_subdomains"
                )
                subdomains = sts[0].get("include_subdomains")
                if not subdomains or not subdomains[0]:
                    return CheckResult.FAILED

                self.evaluated_keys.append("security_headers_config/[0]/strict_transport_security/[0]/preload")
                preload = sts[0].get("preload")
                if not preload or not preload[0]:
                    return CheckResult.FAILED

                self.evaluated_keys.append("security_headers_config/[0]/strict_transport_security/[0]/override")
                override = sts[0].get("override")
                if not override or not override[0]:
                    return CheckResult.FAILED

                return CheckResult.PASSED

        return CheckResult.FAILED


check = CloudFrontResponseHeaderStrictTransportSecurity()



from __future__ import annotations

from typing import Any

from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.terraform.checks.resource.base_resource_negative_value_check import BaseResourceNegativeValueCheck


class DataExplorerSKUHasSLA(BaseResourceNegativeValueCheck):
    def __init__(self) -> None:
        name = "Ensure that data explorer uses Sku with an SLA"
        id = "CKV_AZURE_180"
        supported_resources = ("azurerm_kusto_cluster",)
        categories = (CheckCategories.GENERAL_SECURITY,)
        super().__init__(
            name=name,
            id=id,
            categories=categories,
            supported_resources=supported_resources,
            missing_attribute_result=CheckResult.UNKNOWN,
        )

    def get_inspected_key(self) -> str:
        return "sku/[0]/name"

    def get_forbidden_values(self) -> list[Any]:
        return [
            "Dev(No SLA)_Standard_D11_v2",
            "Dev(No SLA)_Standard_E2a_v4",
        ]


check = DataExplorerSKUHasSLA()



from typing import Any

from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceValueCheck
from checkov.common.models.enums import CheckCategories
from checkov.common.models.consts import ANY_VALUE


class MDBSecurityGroup(BaseResourceValueCheck):
    def __init__(self) -> None:
        name = "Ensure security group is assigned to database cluster."
        id = "CKV_YC_1"
        supported_resources = (
            "yandex_mdb_postgresql_cluster",
            "yandex_mdb_sqlserver_cluster",
            "yandex_mdb_redis_cluster",
            "yandex_mdb_mysql_cluster",
            "yandex_mdb_mongodb_cluster",
            "yandex_mdb_kafka_cluster",
            "yandex_mdb_greenplum_cluster",
            "yandex_mdb_elasticsearch_cluster",
            "yandex_mdb_clickhouse_cluster",
        )
        categories = (CheckCategories.NETWORKING,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self) -> str:
        return "security_group_ids"

    def get_expected_value(self) -> Any:
        return ANY_VALUE


check = MDBSecurityGroup()



from typing import Any

from checkov.common.models.enums import CheckCategories
from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceValueCheck
from checkov.common.models.consts import ANY_VALUE


class ComprehendEntityRecognizerVolumeUsesCMK(BaseResourceValueCheck):
    def __init__(self) -> None:
        name = (
            "Ensure that Comprehend Entity Recognizer's volume is encrypted by KMS using a customer managed Key (CMK)"
        )
        id = "CKV_AWS_268"
        supported_resources = ("aws_comprehend_entity_recognizer",)
        categories = (CheckCategories.ENCRYPTION,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self) -> str:
        return "volume_kms_key_id"

    def get_expected_value(self) -> Any:
        return ANY_VALUE


check = ComprehendEntityRecognizerVolumeUsesCMK()



from __future__ import annotations

import logging
from enum import Enum
from pathlib import Path
from typing import Any, TYPE_CHECKING, overload

from pycep.transformer import BicepElement
from typing_extensions import Literal, TypeAlias

from checkov.bicep.graph_builder.graph_components.block_types import BlockType
from checkov.bicep.graph_builder.graph_components.blocks import BicepBlock
from checkov.bicep.graph_builder.variable_rendering.renderer import BicepVariableRenderer
from checkov.bicep.utils import adjust_value
from checkov.common.graph.graph_builder.graph_components.edge import Edge
from checkov.common.graph.graph_builder.local_graph import LocalGraph
from checkov.common.graph.graph_builder.utils import filter_sub_keys
from checkov.common.util.data_structures_utils import pickle_deepcopy
from checkov.common.util.type_forcers import force_int

if TYPE_CHECKING:
    from checkov.common.graph.graph_builder.graph_components.blocks import Block
    from pycep.typing import (
        BicepJson,
        ResourceAttributes,
        GlobalsAttributes,
        ParameterAttributes,
        VariableAttributes,
        OutputAttributes,
        ModuleAttributes,
    )


BicepElementsAlias: TypeAlias = Literal["globals", "parameters", "variables", "resources", "modules", "outputs"]


class BicepElements(str, Enum):
    GLOBALS: Literal["globals"] = "globals"
    PARAMETERS: Literal["parameters"] = "parameters"
    VARIABLES: Literal["variables"] = "variables"
    RESOURCES: Literal["resources"] = "resources"
    MODULES: Literal["modules"] = "modules"
    OUTPUTS: Literal["outputs"] = "outputs"


class BicepLocalGraph(LocalGraph[BicepBlock]):
    def __init__(self, definitions: dict[Path, BicepJson]) -> None:
        super().__init__()
        self.vertices: list[BicepBlock] = []
        self.definitions = definitions
        self.vertices_by_name: dict[str, int] = {}

    def build_graph(self, render_variables: bool) -> None:
        self._create_vertices()
        logging.info(f"[BicepLocalGraph] created {len(self.vertices)} vertices")
        self._create_edges()
        logging.info(f"[BicepLocalGraph] created {len(self.edges)} edges")

        if render_variables:
            renderer = BicepVariableRenderer(self)
            renderer.render_variables_from_local_graph()

    def _create_vertices(self) -> None:
        for file_path, bicep_conf in self.definitions.items():
            self._create_global_vertices(file_path=file_path, globals_attrs=bicep_conf.get(BicepElements.GLOBALS.value))
            self._create_param_vertices(file_path=file_path, parameters=bicep_conf.get(BicepElements.PARAMETERS.value))
            self._create_var_vertices(file_path=file_path, variables=bicep_conf.get(BicepElements.VARIABLES.value))
            self._create_resource_vertices(file_path=file_path, resources=bicep_conf.get(BicepElements.RESOURCES.value))
            self._create_module_vertices(file_path=file_path, modules=bicep_conf.get(BicepElements.MODULES.value))
            self._create_output_vertices(file_path=file_path, outputs=bicep_conf.get(BicepElements.OUTPUTS.value))

        for i, vertex in enumerate(self.vertices):
            self.vertices_by_block_type[vertex.block_type].append(i)
            self.vertices_block_name_map[vertex.block_type][vertex.name].append(i)
            self.vertices_by_name[vertex.name] = i

            self.in_edges[i] = []
            self.out_edges[i] = []

    def _create_global_vertices(self, file_path: Path, globals_attrs: GlobalsAttributes | None) -> None:
        if not globals_attrs:
            return

        # there can only be one target scope per file
        config = pickle_deepcopy(globals_attrs["scope"])
        attributes = pickle_deepcopy(config)

        self.vertices.append(
            BicepBlock(
                name=BlockType.TARGET_SCOPE,
                config=config,  # type:ignore[arg-type]
                path=str(file_path),
                block_type=BlockType.TARGET_SCOPE,
                attributes=attributes,  # type:ignore[arg-type]
                id=BlockType.TARGET_SCOPE,
            )
        )

    def _create_param_vertices(self, file_path: Path, parameters: dict[str, ParameterAttributes] | None) -> None:
        if not parameters:
            return

        for name, conf in parameters.items():
            config = pickle_deepcopy(conf)
            attributes = pickle_deepcopy(conf)

            self.vertices.append(
                BicepBlock(
                    name=name,
                    config=config,  # type:ignore[arg-type]
                    path=str(file_path),
                    block_type=BlockType.PARAM,
                    attributes=attributes,  # type:ignore[arg-type]
                    id=f"{BlockType.PARAM}.{name}",
                )
            )

    def _create_var_vertices(self, file_path: Path, variables: dict[str, VariableAttributes] | None) -> None:
        if not variables:
            return

        for name, conf in variables.items():
            config = pickle_deepcopy(conf)
            attributes = pickle_deepcopy(conf)

            self.vertices.append(
                BicepBlock(
                    name=name,
                    config=config,  # type:ignore[arg-type]
                    path=str(file_path),
                    block_type=BlockType.VAR,
                    attributes=attributes,  # type:ignore[arg-type]
                    id=f"{BlockType.VAR}.{name}",
                )
            )

    def _create_resource_vertices(self, file_path: Path, resources: dict[str, ResourceAttributes] | None) -> None:
        if not resources:
            return

        for name, conf in resources.items():
            config = pickle_deepcopy(conf)

            attributes: dict[str, Any] = {}
            attributes["decorators"] = pickle_deepcopy(config["decorators"])
            attributes["type_"] = config["type"]
            attributes["api_version_"] = config["api_version"]
            attributes["existing_"] = config["existing"]
            attributes.update(pickle_deepcopy(config["config"]))

            attributes["resource_type"] = config["type"]
            attributes["__start_line__"] = config["__start_line__"]
            attributes["__end_line__"] = config["__end_line__"]

            self.vertices.append(
                BicepBlock(
                    name=name,
                    config=config,  # type:ignore[arg-type]
                    path=str(file_path),
                    block_type=BlockType.RESOURCE,
                    attributes=attributes,
                    id=f"{config['type']}.{name}",
                )
            )

    def _create_module_vertices(self, file_path: Path, modules: dict[str, ModuleAttributes] | None) -> None:
        if not modules:
            return

        for name, conf in modules.items():
            config = pickle_deepcopy(conf)

            attributes: dict[str, Any] = {}
            attributes["decorators"] = pickle_deepcopy(config["decorators"])
            attributes["type_"] = config["type"]
            attributes["detail_"] = config["detail"]
            attributes.update(pickle_deepcopy(config["config"]))

            attributes["resource_type"] = config["type"]
            attributes["__start_line__"] = config["__start_line__"]
            attributes["__end_line__"] = config["__end_line__"]

            self.vertices.append(
                BicepBlock(
                    name=str(name),  # this will be fixed in pycep with the next version, currently type Token
                    config=config,  # type:ignore[arg-type]
                    path=str(file_path),
                    block_type=BlockType.MODULE,
                    attributes=attributes,
                    id=f"{config['type']}.{name}",
                )
            )

    def _create_output_vertices(self, file_path: Path, outputs: dict[str, OutputAttributes] | None) -> None:
        if not outputs:
            return

        for name, conf in outputs.items():
            config = pickle_deepcopy(conf)
            attributes = pickle_deepcopy(conf)

            self.vertices.append(
                BicepBlock(
                    name=name,
                    config=config,  # type:ignore[arg-type]
                    path=str(file_path),
                    block_type=BlockType.OUTPUT,
                    attributes=attributes,  # type:ignore[arg-type]
                    id=f"{BlockType.OUTPUT}.{name}",
                )
            )

    def _create_edges(self) -> None:
        # TODO: support connections in interpolated strings
        for origin_vertex_index, vertex in enumerate(self.vertices):
            for attr_key, attr_value in vertex.attributes.items():
                if isinstance(attr_value, BicepElement):
                    self._create_edge(
                        element_name=attr_value,
                        origin_vertex_index=origin_vertex_index,
                        label=attr_key,
                    )
                if isinstance(attr_value, list):
                    for list_value in attr_value:
                        if isinstance(list_value, BicepElement):
                            self._create_edge(
                                element_name=list_value,
                                origin_vertex_index=origin_vertex_index,
                                label=attr_key,
                            )

    def _create_edge(self, element_name: str, origin_vertex_index: int, label: str) -> None:
        vertex_name = element_name
        if "." in vertex_name:
            # special case for`bicep elements, when properties are accessed
            vertex_name = vertex_name.split(".")[0]

        dest_vertex_index = self.vertices_by_name.get(vertex_name)
        if dest_vertex_index:
            if origin_vertex_index == dest_vertex_index:
                return
            edge = Edge(origin_vertex_index, dest_vertex_index, label)
            self.edges.append(edge)
            self.out_edges[origin_vertex_index].append(edge)
            self.in_edges[dest_vertex_index].append(edge)

    def update_vertices_configs(self) -> None:
        for vertex in self.vertices:
            changed_attributes = list(vertex.changed_attributes.keys())
            changed_attributes = filter_sub_keys(changed_attributes)
            self.update_vertex_config(vertex, changed_attributes)

    @staticmethod
    def update_vertex_config(vertex: Block, changed_attributes: list[str] | dict[str, Any], dynamic_blocks: bool = False) -> None:
        if not changed_attributes:
            # skip, if there is no change
            return

        for attr in changed_attributes:
            new_value = vertex.attributes.get(attr, None)
            if vertex.block_type == BlockType.RESOURCE:
                BicepLocalGraph.update_config_attribute(
                    config=vertex.config["config"], key_to_update=attr, new_value=new_value
                )

    @staticmethod
    def update_config_attribute(config: list[Any] | dict[str, Any], key_to_update: str, new_value: Any) -> None:
        key_parts = key_to_update.split(".")

        if isinstance(config, dict):
            key = key_parts[0]
            if len(key_parts) == 1:
                BicepLocalGraph.update_config_value(config=config, key=key, new_value=new_value)
                return
            else:
                key, key_parts = BicepLocalGraph.adjust_key(config, key, key_parts)
                if len(key_parts) == 1:
                    BicepLocalGraph.update_config_value(config=config, key=key, new_value=new_value)
                    return

                BicepLocalGraph.update_config_attribute(config[key], ".".join(key_parts[1:]), new_value)
        elif isinstance(config, list):
            key_idx = force_int(key_parts[0])
            if key_idx is None:
                return

            if len(key_parts) == 1:
                BicepLocalGraph.update_config_value(config=config, key=key_idx, new_value=new_value)
                return
            else:
                BicepLocalGraph.update_config_attribute(config[key_idx], ".".join(key_parts[1:]), new_value)

        return

    @overload
    @staticmethod
    def update_config_value(config: list[Any], key: int, new_value: Any) -> None:
        ...

    @overload
    @staticmethod
    def update_config_value(config: dict[str, Any], key: str, new_value: Any) -> None:
        ...

    @staticmethod
    def update_config_value(config: list[Any] | dict[str, Any], key: int | str, new_value: Any) -> None:
        new_value = adjust_value(config[key], new_value)  # type:ignore[index]
        if new_value is None:
            # couldn't find key in in value object
            return

        config[key] = new_value  # type:ignore[index]

    @staticmethod
    def adjust_key(config: dict[str, Any], key: str, key_parts: list[str]) -> tuple[str, list[str]]:
        """Adjusts the key, if it consists of multiple dots

        Ex:
        config = {"'container.registry'": "acrName"}
        key = "'container"
        key_parts = ["'container", "registry'"]

        returns new_key = "'container.registry'"
                new_key_parts = ["'container.registry'"]
        """

        if key not in config:
            if len(key_parts) >= 2:
                new_key = ".".join(key_parts[:2])
                new_key_parts = [new_key] + key_parts[2:]

                return BicepLocalGraph.adjust_key(config, new_key, new_key_parts)

        return key, key_parts

    def get_resources_types_in_graph(self) -> list[str]:
        return []



from __future__ import annotations

from typing import Any

from checkov.terraform.checks.resource.base_resource_check import BaseResourceCheck
from checkov.common.models.enums import CheckResult, CheckCategories


class AccessControlGroupOutboundRule(BaseResourceCheck):
    def __init__(self) -> None:
        name = "Ensure no security group rules allow outbound traffic to 0.0.0.0/0"
        id = "CKV_NCP_3"
        supported_resources = ("ncloud_access_control_group_rule",)
        categories = (CheckCategories.NETWORKING,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf: dict[str, list[Any]]) -> CheckResult:
        for outbound in conf.get("outbound", []):
            ip = outbound.get("ip_block")
            if ip == ["0.0.0.0/0"] or ip == ["::/0"]:
                return CheckResult.FAILED
        return CheckResult.PASSED


check = AccessControlGroupOutboundRule()



from __future__ import annotations

import re
from typing import TYPE_CHECKING

from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.dockerfile.base_dockerfile_check import BaseDockerfileCheck

if TYPE_CHECKING:
    from dockerfile_parse.parser import _Instruction

UPDATE_COMMANDS_PATTERN = re.compile(r"\s+(?:--)?update(?!\S)")

install_commands = (
    "install",
    "source-install",
    "reinstall",
    "groupinstall",
    "localinstall",
    "add",
    "upgrade"
)


class UpdateNotAlone(BaseDockerfileCheck):
    def __init__(self) -> None:
        name = "Ensure update instructions are not use alone in the Dockerfile"
        id = "CKV_DOCKER_5"
        supported_instructions = ("RUN",)
        categories = (CheckCategories.APPLICATION_SECURITY,)
        super().__init__(name=name, id=id, categories=categories, supported_instructions=supported_instructions)

    def scan_resource_conf(self, conf: list[_Instruction]) -> tuple[CheckResult, list[_Instruction] | None]:
        update_instructions = []
        update_cnt = 0
        i = 0
        for instruction in conf:
            content = instruction["content"]
            if instruction["instruction"] in self.supported_instructions:

                if "update" in content:
                    # do an initial lookup before using the regex
                    if re.search(UPDATE_COMMANDS_PATTERN, content):
                        update_cnt = update_cnt + 1
                        update_instructions.append(i)
                if any(x in content for x in install_commands):
                    update_cnt = update_cnt - 1
            i = i + 1

        if update_cnt <= 0:
            return CheckResult.PASSED, None
        output = []
        for i in update_instructions:
            output.append(conf[i])

        return CheckResult.FAILED, output


check = UpdateNotAlone()



from checkov.terraform.checks.resource.base_resource_negative_value_check import BaseResourceNegativeValueCheck
from checkov.common.models.enums import CheckCategories


class ALBDesyncMode(BaseResourceNegativeValueCheck):
    def __init__(self):
        """
        NIST.800-53.r5 AC-4(21), NIST.800-53.r5 CA-9(1), NIST.800-53.r5 CM-2
        Classic Load Balancer and ALB should be configured with defensive or strictest desync mitigation mode
        """
        name = "Ensure that ALB is configured with defensive or strictest desync mitigation mode"
        id = "CKV_AWS_328"
        supported_resources = ["aws_lb", "aws_alb", "aws_elb"]
        categories = [CheckCategories.NETWORKING]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self):
        return "desync_mitigation_mode"

    def get_forbidden_values(self):
        return "monitor"


check = ALBDesyncMode()
